// DAVTree.java
// $Id: DAVTreeView.java,v 1.3 2000/12/15 12:40:30 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 2000.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui.webdav;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import org.w3c.jwput.gui.webdav.event.*;

/**
 * @version $Revision: 1.3 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class DAVTreeView extends SelectionEventSupport
    implements SelectionListener 
{
    
    class DAVTreeSelectionListener 
	implements TreeSelectionListener
    {
	public void valueChanged(TreeSelectionEvent ev) {
            TreePath treepath  = ev.getPath();
	    TreeNode tnode     = (TreeNode)treepath.getLastPathComponent();
	    if (tnode instanceof DataNode) {
		((DataNode)tnode).acquireChildren();
		treeModel.nodeStructureChanged(tnode);
	    }
	    SelectionEvent sev = 
		new SelectionEvent(DAVTreeView.this, tnode, treepath);
	    fireSelectionEvent(sev);
	}

	DAVTreeSelectionListener() {}
    }

    class DAVTreeExpansionListener
        implements TreeExpansionListener
    {

        public void treeCollapsed(TreeExpansionEvent ev) { }

        public void treeExpanded(TreeExpansionEvent ev) {
	    TreePath treepath = ev.getPath();
	    Object   node     = treepath.getLastPathComponent();
	    if (node instanceof DataNode) {
		DataNode dnode = (DataNode)node;
		dnode.acquireChildren();
		treeModel.nodeStructureChanged((TreeNode)node);
	    }
        }

        DAVTreeExpansionListener() { }
    }

    protected JTree                  tree               = null;
    protected JScrollPane            jscp               = null;
    protected DefaultMutableTreeNode root               = null;
    protected DefaultTreeModel       treeModel          = null;
    protected TreeSelectionModel     selectionModel     = null;

    // SelectionListener

    public void selectionChanged(SelectionEvent ev) {
	// update selected item
	TreePath path = ev.getPath();
	if (! tree.isExpanded(path)) {
	    tree.expandPath(path);
	}
	tree.setSelectionPath(path);
	tree.makeVisible(path);
	tree.scrollPathToVisible(path);
	tree.treeDidChange();
	SelectionEvent sev = new SelectionEvent(this, ev.getNode(), path);
	fireSelectionEvent(sev);
    }

    public Component getComponent() {
	return jscp;
    }

    public DAVTreeView() {
	root               = new DefaultMutableTreeNode("WinieDAV");
	treeModel          = new DefaultTreeModel(root);
	selectionModel     = new DefaultTreeSelectionModel();
	tree               = new JTree(treeModel);
	jscp               = new JScrollPane(tree);

	int mode = TreeSelectionModel.SINGLE_TREE_SELECTION;
        selectionModel.setSelectionMode(mode);
        tree.setSelectionModel(selectionModel);
	tree.addTreeSelectionListener(new DAVTreeSelectionListener());
        tree.addTreeExpansionListener(new DAVTreeExpansionListener());
	jscp.setPreferredSize(new Dimension(200, 300));

	// add root nodes
	String home = System.getProperty("user.home");
	if (home != null) {
	    File fhome = new File(home);
	    if (fhome.isDirectory()) {
		root.add(new LocalNode(fhome, fhome.getAbsolutePath())); 
	    }
	}
	File[] roots = File.listRoots();
	for (int i = 0 ; i < roots.length ; i++) { 
	    root.add(new LocalNode(roots[i], roots[i].getAbsolutePath())); 
	}

    }

}
