// ByteCountInputStream.java
// $Id: ByteCountInputStream.java,v 1.3 2000/08/17 15:07:28 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
package org.w3c.jwput.tui;

import java.io.IOException;
import java.io.InputStream;

/**
 * A Stream that can be monitored by a ByteCountListener
 * @see ByteCountListener
 * @version $Revision: 1.3 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class ByteCountInputStream extends InputStream {

    InputStream in = null;

    long counter       = 0;
    long step          = 0;
    long nextstep      = 0;
    long countermarked = 0;

    ByteCountListener listener = null;

    public int available()
	throws IOException
    {
	return in.available();
    }

    public void mark(int readlimit) {
	in.mark(readlimit);
	countermarked = counter;
    }

    public boolean markSupported() {
	return in.markSupported();
    }

    public void reset()
	throws IOException
    {
	in.reset();
	this.counter  = countermarked;
	this.nextstep = countermarked+step;
    }

    public int read()
	throws IOException
    {
	counter++;
	check();
	return in.read();
    }

    public int read(byte[] b)
	throws IOException
    {
	int nb = in.read(b);
	counter += nb;
	check();
	return nb;
    }

    public int read(byte[] b,
		    int off,
		    int len)
	throws IOException
    {
	int nb = in.read(b, off, len);
	counter += nb;
	check();
	return nb;
    }

    public long skip(long n)
	throws IOException
    {
	long nb = in.skip(n);
	counter += nb;
	check();
	return nb;
    }

    protected void check() {
	if (counter > nextstep) {
	    if (listener != null)
		listener.bytesRead(counter);
	    while (nextstep < counter)
		nextstep += step;
	}
    }

    public void close() 
    	throws IOException
    {
	if (listener != null)
	    listener.streamClosed();
	in.close();
    }

    public long getBytesRead() {
	return counter;
    }

    public void registerByteCountListener(ByteCountListener listener) {
	this.listener = listener;
    }

    public ByteCountInputStream(InputStream in, long step) {
	super();
	this.in       = in;
	this.step     = step;
	this.nextstep = step;
	this.counter  = 0;
    }
}
