// DAVFileView.java
// $Id: DAVFileView.java,v 1.4 2001/01/30 14:26:03 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 2000.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui.webdav;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

import org.w3c.jwput.gui.webdav.event.*;
import org.w3c.jwput.gui.*;

/**
 * @version $Revision: 1.4 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class DAVFileView extends SelectionEventSupport
    implements SelectionListener 
{

    protected DefaultMutableTreeNode selectedNode = null;
    protected JScrollPane            jscp        = null;
    protected JTable                 table       = null;  
    protected TableSorter            sorter      = null;
    protected Vector                 filesdescr  = null;
    protected Vector                 filesdata    = null;
    protected String[]               columnNames = { 
	"", "Name", "Display", "Type",  "Length", "Last Modified" 
    };

    protected static ImageIcon COL_ICON  = Icons.getIcon("dir.gif");
    protected static ImageIcon FILE_ICON = Icons.getIcon("file.gif");
    
    protected TableModel tableModel = new AbstractTableModel() {
	public Class getColumnClass(int j) {
	    if(filesdescr.size() == 0) {
		return null;
	    } else {
		return getValueAt(0, j).getClass();
	    }
	}

	public int getColumnCount() {
	    return columnNames.length;
	}

	public String getColumnName(int j) {
	    return columnNames[j];
	}

	public int getRowCount() {
	    return filesdescr.size();
	}

	public Object getValueAt(int j, int k) {
	    if (filesdescr.size() == 0) {
		return null;
	    } else {
		return ((Vector)filesdescr.elementAt(j)).elementAt(k);
	    }
	}

	public boolean isCellEditable(int j, int k) {
	    return false;
	}

	public void setValueAt(Object obj, int j, int k) {
	}
    };

    protected MouseListener mouseListener = new MouseAdapter() {
	public void mouseClicked(MouseEvent ev) {
	    if (ev.getClickCount() == 2) {
		doubleClick(ev);
            }
	}
    };

    protected void doubleClick(MouseEvent ev) {
	Point point = ev.getPoint();
        int   i     = table.rowAtPoint(point);
	if (i != -1) {
	    Boolean col = (Boolean) table.getValueAt(i, 0);
	    if (col != null && col.booleanValue()) {
		// ok that's a collection, open it
		int max = table.getSelectionModel().getMaxSelectionIndex();
		if (max != -1) {
		    int      row  = sorter.getUnsortedRow(max);
		    FileData data = (FileData)filesdata.elementAt(row);
		    if (data.hasNode()) {
			DefaultMutableTreeNode node = 
			    (DefaultMutableTreeNode)data.getNode();
			TreePath               path = 
			    new TreePath(node.getPath());
			SelectionEvent         sev = 
			    new SelectionEvent(this, node, path);
			fireSelectionEvent(sev);
		    }
		}
	    }
	}
    }

    protected void updateView(Vector v) {
	this.filesdata  = v;
	this.filesdescr = new Vector();
	
	int len = filesdata.size();
	for (int i = 0 ; i < len ; i++) { 
	    FileData data = (FileData) filesdata.elementAt(i);
	    Vector row = new Vector();
	    row.addElement(new Boolean(data.isCollection));
	    row.addElement(data.name);
	    row.addElement(data.display);
	    row.addElement(data.type);
	    row.addElement(new Long(data.length));
	    row.addElement(new Date(data.lastmodified));
	    filesdescr.addElement(row);
	}
	TableModelEvent ev = new TableModelEvent(tableModel);
        synchronized(this) {
            sorter.tableChanged(ev);
            sorter.sortByColumn(1, true);
        }
        table.updateUI();
    }

    // SelectionListener

    public void selectionChanged(SelectionEvent ev) {
	// update table
	selectedNode = (DefaultMutableTreeNode) ev.getNode();
	if (selectedNode instanceof DataNode) {
	    DataNode datanode = (DataNode)selectedNode;
	    FileData fdata    = datanode.getFileData();
	    updateView(fdata.children);
	}
    }

    public Component getComponent() {
	return jscp;
    }

    protected void buildTable(JTable table) {
	table.setShowGrid(false);
	table.updateUI();
	table.setIntercellSpacing(new Dimension(0, 0));
	TableColumn column = table.getColumn(columnNames[0]);
	column.setMaxWidth(25);
	column.setMinWidth(25);
	DefaultTableCellRenderer cr = new DefaultTableCellRenderer() {
	    public void setValue(Object obj) {
		boolean col = (new Boolean(obj.toString())).booleanValue();
		if (col) {
		    setIcon(COL_ICON);
		} else {
		    setIcon(FILE_ICON);
		}
            }

	};
	column.setCellRenderer(cr);
	for (int i = 1 ; i < 5 ; i++) {
	    column = table.getColumn(columnNames[i]);
	    column.setCellRenderer(new DefaultTableCellRenderer());
	}
        table.addMouseListener(mouseListener);
    }

    public DAVFileView() {
	this.filesdescr = new Vector();
	this.sorter     = new TableSorter(tableModel);
	this.table      = new JTable(sorter);
	JPanel p = new JPanel(new BorderLayout());
	p.add(table, BorderLayout.CENTER);
	JPanel whitep = new JPanel();
	whitep.setBackground(Color.white);
	p.add(whitep, BorderLayout.SOUTH);
	this.jscp       = new JScrollPane(p);
	jscp.setPreferredSize(new Dimension(400, 200));
	jscp.setOpaque(true);
	buildTable(table);
    }
    
}
